#!/usr/bin/perl

my $logger = 0;

use Data::Dumper;
use CGI::Session;
use CGI::Carp  qw/ fatalsToBrowser /;
use XML::DOM;

my $XML_UNSUPPORTED = "Unsupported method";
my $XML_BADPOST = "error parsing POST data";
my $XML_BADROOT = "Invalid request";
my $XML_INVALIDSESSION = "Invalid session info";
my $XML_UNKNOWN = "Unknown error while processing your request";
my $XML_BADIP = "Connection denied";

require("parameters_inc.pl");
require("oai_functions.pl");
require("oaicommon.pl");
require("oaidb.pl");
require("access.pl");
if (!ConnectionAllowed($ENV{"REMOTE_ADDR"}, $allowManagerConnect)) {
  XMLError($XML_BADIP);
  exit;
}

my $XMLHeaderSent = 0;
my $_SESSION;
my $parser;
my $dom;
my $SessionProperties = {Directory=>"/tmp"};
my $ob_data, $oldout;

sub session_start {
}

sub session_id {
}

sub capture {
  open $oldout, ">&STDOUT"    or die "Can't dup STDOUT: $!";
  open OLDERR, ">&", \*STDERR or die "Can't dup STDERR: $!";

  close STDOUT;
  open STDOUT, '>', \$ob_data or die "Can't redirect STDOUT: $!";
  close STDERR;
  open STDERR, ">&STDOUT"     or die "Can't dup STDOUT: $!";

  select STDERR; $| = 1;  # make unbuffered
  select STDOUT; $| = 1;  # make unbuffered

  # print STDOUT "stdout 1\n";  # this works for
  # print STDERR "stderr 1\n";      # subprocesses too
  for $sig ('INT', 'TERM', 'HUP', 'QUIT', 'TSTP', __DIE__) { $SIG{$sig} = \&uncapture; }
}

sub logger {
  my $data = shift;

  if ($logger) {
    open(LOG, ">>" . sprintf("manager-%s.log", strftime("%Y-%m-%d", gmtime(time))));
    foreach my $val (split(/\n/, $data)) {
      if ($val ne "") {
        printf LOG "%s %s\n", ISO8601Date(), $val;
      }
    }
    close(LOG);
  }
  return $data;
}

sub uncapture {
  for $sig ('INT', 'TERM', 'HUP', 'QUIT', 'TSTP', __DIE__) { $SIG{$sig} = 'DEFAULT'; }

  open STDOUT, ">&", $oldout or die "Can't dup \$oldout: $!";
  open STDERR, ">&OLDERR"    or die "Can't dup OLDERR: $!";
  print $ob_data;
  logger($ob_data);
  # print STDOUT "stdout 2\n";
  # print STDERR "stderr 2\n";
}

sub XMLHeader {
  if ($XMLHeaderSent != 1) {
    $XMLHeaderSent = 1;
    print("Content-Type: text/xml;charset=utf-8\n\n");
    print "<" . "?xml version = \"1.0\" encoding = \"utf-8\"?" . ">\n";
  }
}

sub XMLError {
  my $errstr = shift;
  my $rootitem = shift;
  $rootitem = "error" if (!$rootitem);
  XMLHeader();
  print XMLtag($rootitem, {"errorCode" => 1, "errorMessage" => $errstr});
  uncapture() if ($logger);
  exit;
}

sub ReadInput {
  $data_length = $ENV{'CONTENT_LENGTH'};
  $bytes_read = read(STDIN, $data, $data_length);
  return $data;
}

sub StartSession {
  my $domroot = shift;

  $_SESSION = new CGI::Session(undef, getSubelementContent($domroot, "sessionID"), $SessionProperties);
  if (!$manager->{$_SESSION->param("user")} || ($manager->{$_SESSION->param("user")} ne $_SESSION->param("password"))) {
    XMLError($XML_INVALIDSESSION);
  }
}

sub SetMinMax {
  my $set = shift;
  my $min = shift;
  my $max = shift;

  if ($set gt $$max) {
    $$max = $set;
  }
  if ($set lt $$min) {
    $$min = $set;
  }
}

sub IsInRange {
  my ($what, $in_array) = @_;
  if ((!defined($in_array->{"start"}) || ($in_array->{"start"} le $what)) &&
      (!defined($in_array->{"end"})   || ($in_array->{"end"}   ge $what))) {
    return 1;
  } else {
    return 0;
  }
}

sub IsInRangeNum {
  my ($what, $in_array) = @_;
  if ((!defined($in_array->{"start"}) || ($in_array->{"start"} <= $what)) &&
      (!defined($in_array->{"end"})   || ($in_array->{"end"}   >  $what))) {
    return 1;
  } else {
    return 0;
  }
}

sub max {
  my $max = shift(@_);
  foreach $foo (@_) {
    $max = $foo if $max < $foo;
  }
  return $max;
}

sub RulesetFileInfo {
  if (-r "MetaEditorRuleset.xml") {
    return XMLtag("rulesetFile", NULL, ISO8601Date(max((stat "MetaEditorRuleset.xml")[9], (stat "MetaEditorRuleset.xml")[10])));
  } else {
    return "";
  }
}

sub ManagerLogin {
  my $domroot = shift;

  $domuser = getSubelementContent($domroot, "user");
  $dompassword = getSubelementContent($domroot, "password");
  if ($manager->{$domuser} ne $dompassword) {
    XMLHeader();
    print XMLtag("login", undef, XMLtag("result", undef, 0));
  } else {
    $_SESSION = new CGI::Session(undef, undef, $SessionProperties);
    $_SESSION->expire('1h');
    $_SESSION->param("user", $domuser);
    $_SESSION->param("password", $dompassword);
    XMLHeader();
    print XMLtag("login", undef, XMLtag("result", undef, 1) . XMLtag("sessionID", undef, $_SESSION->id()) .
      RulesetFileInfo());
  }
  return 0;
}

sub ManagerListSchemas {
  my $domroot = shift;

  StartSession($domroot);
  XMLHeader();
  $records = getRecordsNew($repository);
  my $data;
  foreach $key (sort keys %$revmetadataFormats) {
    #$data->{$key} = array();
    $data->{$key}->{"root_name"} = $key;
    $data->{$key}->{"createdmin"} = "9999-12-31T23:59:59Z";
    $data->{$key}->{"createdmax"} = "0001-01-01T00:00:00Z";
    $data->{$key}->{"modifiedmin"} = "9999-12-31T23:59:59Z";
    $data->{$key}->{"modifiedmax"} = "0001-01-01T00:00:00Z";
    $data->{$key}->{"deletedmin"} = "9999-12-31T23:59:59Z";
    $data->{$key}->{"deletedmax"} = "0001-01-01T00:00:00Z";
    $data->{$key}->{"recordCount"} = 0;
    $data->{$key}->{"deletedRecordCount"} = 0;
  }
  foreach $id (keys %$records) {
    foreach $root_name (sort keys %{$records->{$id}}) {
      $onerecord = $records->{$id}->{$root_name};
      if ($onerecord->{"created"} ne $onerecord->{"deleted"}) {
        $data->{$root_name}->{"deletedRecordCount"}++;
      } else {
        $data->{$root_name}->{"recordCount"}++;
      }
      SetMinMax($onerecord->{"created"}, \$data->{$root_name}->{"createdmin"}, \$data->{$root_name}->{"createdmax"});
      SetMinMax($onerecord->{"modified"}, \$data->{$root_name}->{"modifiedmin"}, \$data->{$root_name}->{"modifiedmax"});
      SetMinMax($onerecord->{"deleted"}, \$data->{$root_name}->{"deletedmin"}, \$data->{$root_name}->{"deletedmax"});
    }
  }
  print XMLtagStart("listSchemas", {"errorCode" => 0});
  foreach $key (keys %$data) {
    $value = $data->{$key};
    print XMLtagStart("schema");

    print XMLtag("recordCount", NULL, $value->{"recordCount"});
    print XMLtag("deletedRecordCount", NULL, $value->{"deletedRecordCount"});
    print XMLtag("type", NULL, $revmetadataFormats->{$key}->{"rootElementName"});
    print XMLtag("namespaceURI", NULL, $revmetadataFormats->{$key}->{"namespace"});
    print XMLtag("schemaLocation", NULL, $revmetadataFormats->{$key}->{"schema"});
    if (($value->{"recordCount"} != 0) || ($value->{"deletedRecordCount"} != 0)) {
      print XMLtag("creationInterval", {"start" => $value->{"createdmin"}, "end" => $value->{"createdmax"}});
      print XMLtag("modificationInterval", {"start" => $value->{"modifiedmin"}, "end" => $value->{"modifiedmax"}});
      print XMLtag("deletionInterval", {"start" => $value->{"deletedmin"}, "end" => $value->{"deletedmax"}});
    } else {
      print XMLtag("creationInterval");
      print XMLtag("modificationInterval");
      print XMLtag("deletionInterval");
    }

    print XMLtagEnd("schema");
  }
  print XMLtagEnd("listSchemas");
  return 0;
}

sub ManagerGetRecords {
  my $domroot = shift;

  StartSession($domroot);
  XMLHeader();

  @domfilter = $domroot->getElementsByTagName("filter");
  if ($#domfilter < 0) {
    XMLError($XML_BADROOT);
  }
  @domfilterarray = $domfilter[0]->getChildNodes();
  #my $filter;
  #my $filter->{"excludeType"};
  #my $filter->{"identifier"};
  #my $filter->{"created"};
  #my $filter->{"modified"};
  #my $filter->{"deleted"};
  #my $filter->{"rows"};
  $filter->{"needDeleted"} = 0;
  for $value (@domfilterarray) {
    if ($value->getNodeType == ELEMENT_NODE) {
      $filter->{"excludeType"}->{$value->getFirstChild()->toString()} = 1 if ($value->getTagName() eq "excludeType");
      $filter->{"identifier"}->{"start"} = $value->getAttribute("start") if ($value->getTagName() eq "identifier");
      $filter->{"identifier"}->{"end"} = $value->getAttribute("end") if ($value->getTagName() eq "identifier");
      $filter->{"created"}->{"start"} = $value->getAttribute("start") if ($value->getTagName() eq "creation");
      $filter->{"created"}->{"end"} = $value->getAttribute("end") if ($value->getTagName() eq "creation");
      $filter->{"modified"}->{"start"} = $value->getAttribute("start") if ($value->getTagName() eq "modification");
      $filter->{"modified"}->{"end"} = $value->getAttribute("end") if ($value->getTagName() eq "modification");
      $filter->{"deleted"}->{"start"} = $value->getAttribute("start") if ($value->getTagName() eq "deletion");
      $filter->{"deleted"}->{"end"} = $value->getAttribute("end") if ($value->getTagName() eq "deletion");
      $filter->{"needDeleted"} = 1 if ($value->getTagName() eq "deletion");
      $filter->{"rows"}->{"start"} = $value->getAttribute("start") if ($value->getTagName() eq "rows");
      $filter->{"rows"}->{"end"} = $value->getAttribute("end") if ($value->getTagName() eq "rows");
    }
  }
  $records = getRecordsNew($repository);
  $recordCount = 0;
  foreach $id (keys %$records) {
    foreach $root_name (keys %{$records->{$id}}) {
      $onerecord = $records->{$id}->{$root_name};
      $oai_id = "oai:" . $config->{"repositoryId"} . ":" . $onerecord->{"id"};
      if (!defined($filter->{"excludeType"}->{$root_name})) {
        if (IsInRange($oai_id, $filter->{"identifier"}) &&
            IsInRange($onerecord->{"created"}, $filter->{"created"}) &&
            IsInRange($onerecord->{"modified"}, $filter->{"modified"}) &&
            IsInRange($onerecord->{"deleted"}, $filter->{"deleted"}) &&
            (($onerecord->{"deleted"} eq $onerecord->{"created"}) || $filter->{"needDeleted"}) ) {
          $recordCount++;
        }
      }
    }
  }
  # copy-paste programming - yay \o/
  print XMLtagStart("getRecords", {"recordCount" => $recordCount, "errorCode" => 0});
  $rownum = 0;
  foreach $id (keys %$records) {
    foreach $root_name (keys %{$records->{$id}}) {
      $onerecord = $records->{$id}->{$root_name};
      $oai_id = "oai:" . $config->{"repositoryId"} . ":" . $onerecord->{"id"};
      if (!defined($filter->{"excludeType"}->{$root_name})) {
        if (IsInRange($oai_id, $filter->{"identifier"}) &&
            IsInRange($onerecord->{"created"}, $filter->{"created"}) &&
            IsInRange($onerecord->{"modified"}, $filter->{"modified"}) &&
            IsInRange($onerecord->{"deleted"}, $filter->{"deleted"}) &&
            (($onerecord->{"deleted"} eq $onerecord->{"created"}) || $filter->{"needDeleted"}) ) {
          if (IsInRangeNum($rownum, $filter->{"rows"})) {
            print XMLtagStart("record");
            print XMLtag("identifier", NULL, $oai_id);
            print XMLtag("creationDate", NULL, $onerecord->{"created"});
            print XMLtag("modificationDate", NULL, $onerecord->{"modified"});
            if ($onerecord->{"created"} ne $onerecord->{"deleted"}) {
              print XMLtag("deletionDate", NULL, $onerecord->{"deleted"});
            } else {
              print XMLtag("deletionDate");
            }
            print XMLtag("content", NULL, $onerecord->{"xml"});
            print XMLtagEnd("record");
          }
          $rownum++;
        }
      }
    }
  }
  print XMLtagEnd("getRecords");
}

#
sub MakeXML {
  my $work = shift;
  if (defined($work)) {
    my @x = $work->getElementsByTagName("content");
    if ($#x >= 0) {
      $data = $x[0]->toString();
      $data =~ s/^<content>//;
      $data =~ s/<\/content>$//;
      return $data;
    }
  }
  return 0;
}

sub ManagerCommitCreate {
  my ($db, $work) = @_;

  $root_namespace = getSubelementContent($work, "namespaceURI");
  $root_name = undef;
  foreach $format_name (keys %$metadataFormats) {
    if ($root_namespace eq $metadataFormats->{$format_name}->{"namespace"}) {
      $root_name = $metadataFormats->{$format_name}->{"rootElementName"};
    }
  }
  if ($root_name) {
	$schemalist{$root_name} = 1;
    my $data = MakeXML($work);
    $maxid++;
    $mydate = ISO8601Date();
    $result = WriteRecord($db, $repository->{"table"}, {"id" => $maxid, "sett" => "",
      "created" => $mydate, "modified" => $mydate, "deleted" => $mydate,
      "xml" => $data, "harvestable" => ynHarvestable($data),
      "root_name" => $root_name, "root_namespace" => $root_namespace});
    return ($result ? 1 : 0);
  } else {
    return 0;
  }
}

sub ManagerCommitModify {
  my ($db, $work) = @_;

  my $data = MakeXML($work);
  if($data =~ /^\s*<[^:]+:([^\s]+)/i) {
    $schemalist{$1} = 1;
  }

  # TODO:HACK: "'" . <blabla> . "'"
  $result = $db->update($repository->{"table"}, {"modified" => $db->escapedate(ISO8601Date()), "xml" => $db->escapestr($data), "harvestable" => $db->escapestr(ynHarvestable($data))}, "id = '" . $db->escapestr(ExplodeID(getSubelementContent($work, "identifier"))) . "'");

  return ($result ? 1 : 0);
}

sub ManagerCommitDelete {
  my ($db, $work) = @_;

  # TODO:HACK: "'" . <blabla> . "'"
  if ($config->{"deletedRecord"} eq "persistent") {
    $result = $db->delete($repository->{"table"}, "id = " . $db->escapestr(ExplodeID(getSubelementContent($work, "identifier"))));
  } else {
    $result = $db->update($repository->{"table"}, {"deleted" => $db->escapedate(ISO8601Date())}, "id = '" . $db->escapestr(ExplodeID(getSubelementContent($work, "identifier"))) . "'");
  }

  return ($result ? 1 : 0);
}

sub ManagerCommitChanges {
  my $domroot = shift;

  StartSession($domroot);
  XMLHeader();

  @domroot = $domroot->getChildNodes();
  @domchangearray = $domroot[0]->getChildNodes();
  my $NR = 0;
  my $EC = 0;

  my $db = new transdb($repository->{"type"}, $repository->{"host"}, $repository->{"user"}, $repository->{"password"}, $repository->{"database"});
  $db->ErrorReporting($repository->{"warn"});
  getRecordsNew($repository, 0, 0);
  for my $work (@domchangearray) {
    if ($work->getNodeType == ELEMENT_NODE) {
      if ($work->getTagName() eq "create") {
        $NR++;
        $EC += ManagerCommitCreate($db, $work);
      } elsif ($work->getTagName() eq "modify") {
        $NR++;
        $EC += ManagerCommitModify($db, $work);
      } elsif ($work->getTagName() eq "delete") {
        $NR++;
        $EC += ManagerCommitDelete($db, $work);
      }
    }
  }
  $db->SQL_disconnect();
  triggerHarvester(keys %schemalist);
  if ($NR != $EC) {
    XMLError($NR . "/" . $EC . " error(s)", "commitChanges");
  } else {
    print XMLtag("commitChanges", {"errorCode" => 0});
  }
}

sub ManagerGetRuleset {
  my $domroot = shift;

  StartSession($domroot);
  XMLHeader();

  if (-r "MetaEditorRuleset.xml") {
    print XMLtagStart("getRuleset", {"errorCode" => 0});
    open(FP, "<MetaEditorRuleset.xml");
    while ($line = <FP>) {
      print $line;
    }
    close(FP);
    print XMLtagEnd("getRuleset");
  } else {
    XMLError("Cannot open file", "getRuleset");
  }
}

sub main {
  umask 0027;

  capture() if ($logger);

  if ($ENV{"REQUEST_METHOD"} ne "POST") {
    XMLError($XML_UNSUPPORTED);
  } else {
    $input = ReadInput();
    logger("### INPUT ###");
    logger($input);
    logger("### OUTPUT ###");
    $parser = new XML::DOM::Parser;
    $dom = $parser->parse($input);
    if (! $dom) {
      XMLError($XML_BADPOST);
    } else {
      my @domrootnodes = $dom->getChildNodes();
      my $teh_request = $domrootnodes[0]->getTagName();
      my $r = -1;
      $r = ManagerLogin($dom) if ($teh_request eq "login");
      $r = ManagerListSchemas($dom) if ($teh_request eq "listSchemas");
      $r = ManagerGetRecords($domrootnodes[0]) if ($teh_request eq "getRecords");
      $r = ManagerCommitChanges($dom) if ($teh_request eq "commitChanges");
      $r = ManagerGetRuleset($dom) if ($teh_request eq "getRuleset");
      XMLError($XML_BADROOT) if ($r == -1);
    }
  }
}

main();
uncapture() if ($logger);
