#!/usr/bin/perl
package transdb;

use Data::Dumper;
use DBI;

use vars qw($AUTOLOAD $VERSION);

$VERSION = "0.01";

sub SQL_connect {
  my $this = shift;

  #my %drivers;
  #for (DBI->available_drivers) { $drivers{$_} = 1 }

  if ($this->{'type'} eq 'csv') {
    $this->{'db'} = DBI->connect("DBI:CSV:");
  } elsif ($this->{'type'} eq 'mysql') {
    $this->{'db'} = DBI->connect(sprintf('DBI:mysql:database=%s;host=%s', $this->{'database'}, $this->{'host'}), $this->{'user'}, $this->{'password'})
  } elsif (($this->{'type'} eq 'pgsql') || ($this->{'type'} eq 'psql')) {
    $this->{'type'} = 'psql';
    $this->{'db'} = DBI->connect(sprintf('DBI:Pg:dbname=%s;host=%s', $this->{'database'}, $this->{'host'}), $this->{'user'}, $this->{'password'});
  } elsif (($this->{'type'} eq 'interbase') || ($this->{'type'} eq 'firebird')) {
    $this->{'type'} = 'interbase';
    $this->{'db'} = DBI->connect(sprintf('DBI:InterBase:dbname=%s;host=%s', $this->{'database'}, $this->{'host'}), $this->{'user'}, $this->{'password'});
    $this->{'db'}->{LongReadLen} = 32768;
    $this->{'db'}->{ib_timestampformat} = '%Y-%m-%dT%H:%M:%SZ';
    #$this->{'db'}->{ib_dateformat} = '%m-%d-%Y';
    #$this->{'db'}->{ib_timeformat} = '%H:%M';
  } else {
    print "ismeretlen adatbazistipus: " . $this->{'type'} . "\n";
    $this->{'type'} = 'none';
  }
  if (!defined($this->{'db'})) {
    $this->{'type'} = 'none';
  }
}

sub SQL_disconnect {
  my $this = shift;
  if ($this->{'db'}) {
    $this->{'db'}->disconnect;
  } else {
    $this->SQLFailed("disconnect", "")
  }
  return undef;
}

sub new {
  my ($this, $type, $host, $user, $password, $database, $defaultstructure, $separator) = @_;
  my $this = {};
  $this->{'type'} = $type;
  $this->{'host'} = $host;
  $this->{'user'} = $user;
  $this->{'password'} = $password;
  $this->{'database'} = $database;
  $this->{'defaultstructure'} = $defaultstructure;
  $this->{'separator'} = $separator;
  $this->{'er'} = 1;
  SQL_connect($this);
  bless ($this);
}

# on/off == 0/1
sub ErrorReporting {
  my $this = shift;
  $this->{'er'} = shift;
}

sub SQLFailed {
  my ($this, $method, $table) = @_;
  if ($this->{"er"} != 0) {
    printf("sql statement failed %s/%s/%s/(%s)/%s/%s\n", $this->{"type"}, $this->{"host"}, $this->{"user"}, $this->{"database"}, $method, $table);
  }
}

sub selectall {
  my $this = shift;
  @_[1] = "" if (!defined(@_[1]));
  @_[2] = -1 if (!defined(@_[2]));
  @_[3] = -1 if (!defined(@_[3]));
  my @x;
  my $defaultstructure = 'id,sett,created,modified,deleted,xml,harvestable,root_name,root_namespace,externalid';

  if ($this->{'db'}) {
    return $this->mysql_selectall(@_) if ($this->{'type'} eq 'mysql');
    return $this->psql_selectall(@_) if ($this->{'type'} eq 'psql');
    return $this->interbase_selectall(@_) if ($this->{'type'} eq 'interbase');
    if ($this->{'type'} eq 'csv') {
       if (! (-e @_[0])) {
         open CSV, ">" . @_[0];
         chomp($defaultstructure);
         print CSV $defaultstructure;
         print CSV "\r\n";
         close CSV;
       }
      @x = $this->mysql_selectall(@_);
      return @x;
    }
  }
  $this->SQLFailed("selectall", $table) if (($this->{"er"} == 1) && (!@x));
  return -1 if (!@x);
  return @x;
}

sub update {
  my $this = shift;
  @_[1] = "" if (!defined(@_[1]));
  my $x;

  if ($this->{'db'}) {
    $x = $this->mysql_update(@_) if ($this->{'type'} eq 'mysql');
    $x = $this->psql_update(@_) if ($this->{'type'} eq 'psql');
    $x = $this->mysql_update(@_) if ($this->{'type'} eq 'csv');
    $x = $this->mysql_update(@_) if ($this->{'type'} eq 'interbase');
  }
  $this->SQLFailed("update", $table) if (($this->{"er"} == 1) && (!$x));
  return $x;
}

sub delete {
  my $this = shift;
  @_[1] = "" if (!defined(@_[1]));
  my $x;

  if ($this->{'db'}) {
    $x = $this->mysql_delete(@_) if ($this->{'type'} eq 'mysql');
    $x = $this->mysql_delete(@_) if ($this->{'type'} eq 'psql');
    $x = $this->mysql_delete(@_) if ($this->{'type'} eq 'csv');
    $x = $this->mysql_delete(@_) if ($this->{'type'} eq 'interbase');
  }
  $this->SQLFailed("delete", $table) if (($this->{"er"} == 1) && (!$x));
  return $x;
}

sub insert {
  my $this = shift;
  @_[1] = "" if (!defined(@_[1]));
  my $x;

  if ($this->{'db'}) {
    $x = $this->mysql_insert(@_) if ($this->{'type'} eq 'mysql');
    $x = $this->mysql_insert(@_) if ($this->{'type'} eq 'psql');
    $x = $this->mysql_insert(@_) if ($this->{'type'} eq 'csv');
    $x = $this->mysql_insert(@_) if ($this->{'type'} eq 'interbase');
  }
  $this->SQLFailed("insert", $table) if (($this->{"er"} == 1) && (!$x));
  return $x;
}

sub escapestr {
  my $this = shift;
  return $this->mysql_escapestr(@_) if ($this->{'type'} eq 'mysql');
  return $this->psql_escapestr(@_) if ($this->{'type'} eq 'psql');
  return $this->csv_escapestr(@_) if ($this->{'type'} eq 'csv');
  return $this->interbase_escapestr(@_) if ($this->{'type'} eq 'interbase');
  $this->SQLFailed("escapestr", "");
  return @_[0];
}

sub escapedate {
  my $this = shift;
  return $this->mysql_escapedate(@_) if ($this->{'type'} eq 'mysql');
  return $this->psql_escapedate(@_) if ($this->{'type'} eq 'psql');
  return $this->csv_escapedate(@_) if ($this->{'type'} eq 'csv');
  return $this->interbase_escapedate(@_) if ($this->{'type'} eq 'interbase');
  $this->SQLFailed("escapedate", "");
  return @_[0];
}

sub unescapestr {
  my $this = shift;
  return $this->mysql_unescapestr(@_) if ($this->{'type'} eq 'mysql');
  return $this->psql_unescapestr(@_) if ($this->{'type'} eq 'psql');
  return $this->csv_unescapestr(@_) if ($this->{'type'} eq 'csv');
  return $this->interbase_unescapestr(@_) if ($this->{'type'} eq 'interbase');
  $this->SQLFailed("unescapestr", "");
  return @_[0];
}

sub unescapedate {
  my $this = shift;
  return $this->mysql_unescapedate(@_) if ($this->{'type'} eq 'mysql');
  return $this->psql_unescapedate(@_) if ($this->{'type'} eq 'psql');
  return $this->csv_unescapedate(@_) if ($this->{'type'} eq 'csv');
  return $this->interbase_unescapedate(@_) if ($this->{'type'} eq 'interbase');
  $this->SQLFailed("unescapedate", "");
  return @_[0];
}







### ganyolas!

sub interbase_escapestr {
  my $this = shift;
  return @_[0];
}

sub interbase_escapedate {
  my $this = shift;
  @_[0] =~ s/([0-9]{4}-[0-9]{2}-[0-9]{2})T([0-9]{2}:[0-9]{2}:[0-9]{2})Z/$1 $2/;
  return @_[0];
}

sub interbase_unescapestr {
  my $this = shift;
  return @_[0];
}

sub interbase_unescapedate {
  my $this = shift;
  return @_[0];
  #@_[0] =~ s@([0-9]{2})/([0-9]{2})/([0-9]{4}) ([0-9]{2}:[0-9]{2}:[0-9]{2})@$3-$1-$2T$4Z@;
}

sub interbase_selectall {
  my($this, $table, $where, $limit, $offset) = @_;

  my $query = "SELECT * FROM " . $table . "";
  if ($where ne "") {
    $query .= " WHERE " . $where;
  }
  if ($limit != -1) {
    print "HIBA\n";
    #$query .= " LIMIT " . $limit;
  }
  if ($offset != -1) {
    print "HIBA\n";
    #$query .= " OFFSET " . $offset;
  }
  my $sth = $this->{'db'}->prepare($query);
  my $rv = $sth->execute;
  return -1 if (!defined($rv));
  my @result;
  while (my $row = $sth->fetchrow_hashref) {
    my $row2, $tmp;
    foreach $tmp (keys %{$row}) {
      $row2->{lc($tmp)} = $row->{$tmp};
    }
    push @result, $row2;
  }
  return @result;
}



sub csv_escapestr {
  my $this = shift;
  return @_[0];
}

sub csv_escapedate {
  my $this = shift;
  return @_[0];
}

sub csv_unescapestr {
  my $this = shift;
  return @_[0];
}

sub csv_unescapedate {
  my $this = shift;
  return @_[0];
}

sub csv_update {
  my $this = shift;
  return undef;
}


sub mysql_escapestr {
  my $this = shift;
  return @_[0];
}

sub mysql_escapedate {
  my $this = shift;
  return @_[0];
}

sub mysql_unescapestr {
  my $this = shift;
  return @_[0];
}

sub mysql_unescapedate {
  my $this = shift;
  my $x = @_[0];
  @_[0] =~ s/([0-9]{4}-[0-9]{2}-[0-9]{2}) ([0-9]{2}:[0-9]{2}:[0-9]{2})/$1T$2Z/;
  return @_[0];
}

sub mysql_unescapeint {
  my $this = shift;
  return @_[0];
}

sub mysql_selectall {
  my($this, $table, $where, $limit, $offset) = @_;

  my $query = "SELECT * FROM " . $table . "";
  if ($where ne "") {
    $query .= " WHERE " . $where;
  }
  if ($limit != -1) {
    if ($offset != -1) {
      $query .= " LIMIT " . $offset . "," . $limit;
    } else {
      $query .= " LIMIT " . $limit;
    }
  }
  my $sth = $this->{'db'}->prepare($query);
  my $rv = $sth->execute;
  return -1 if (!defined($rv));
  my @result;
  while (my $row = $sth->fetchrow_hashref) {
    push @result, $row;
  }
  return @result;
}

sub mysql_update {
  my ($this, $table, $set_array, $where) = @_;

  $n = 0;
  $query = "UPDATE " . $table . " SET ";
  my @vals;
  foreach $key (sort keys %$set_array) {
    if ($n > 0) {
      $query .= ", ";
    }
    $query .= "" . $key . "=?";
    push @vals, $set_array->{$key};
    $n++;
  }
  if ($where ne "") {
    $query .= " WHERE " . $where;
  }
  my $sth = $this->{'db'}->prepare($query);
  return $sth->execute(@vals);
}

sub mysql_delete {
  my ($this, $table, $where) = @_;

  my $query = "DELETE FROM " . $table . "";
  if ($where ne "") {
    $query .= " WHERE " . $where;
    #$this->SQLFailed("delete", $table);
  }
  if ($sth = $this->{'db'}->prepare($query)) {
    return $sth->execute;
  }
}

sub mysql_insert {
  my ($this, $table, $set_array) = @_;
  my $n = 0;
  my $cols = "";
  my @vals;
  my $valjoker = "";
  foreach $key (sort keys %$set_array) {
    if ($n > 0) {
      $cols .= ", ";
      $valjoker .= ", ";
    }
    $cols .= $key;
    push @vals, $set_array->{$key};
    $valjoker .= "?";
    $n++;
  }
  if ($sth = $this->{'db'}->prepare(sprintf("INSERT INTO %s (%s) VALUES (%s)", $table, $cols, $valjoker))) {
    return $sth->execute(@vals);
  }
}


sub psql_escapestr {
  my $this = shift;
  return @_[0];
}

sub psql_escapedate {
  my $this = shift;
  return @_[0];
}

sub psql_unescapestr {
  my $this = shift;
  return @_[0];
}

sub psql_unescapedate {
  my $this = shift;
  my $x = @_[0];
  @_[0] =~ s/([0-9]{4}-[0-9]{2}-[0-9]{2}) ([0-9]{2}:[0-9]{2}:[0-9]{2})/$1T$2Z/;
  return @_[0];
}

sub psql_selectall {
  my($this, $table, $where, $limit, $offset) = @_;

  my $query = "SELECT * FROM " . $table . "";
  if ($where ne "") {
    $query .= " WHERE " . $where;
  }
  if ($limit != -1) {
    $query .= " LIMIT " . $limit;
  }
  if ($offset != -1) {
    $query .= " OFFSET " . $offset;
  }
  my $sth = $this->{'db'}->prepare($query);
  my $rv = $sth->execute;
  return -1 if (!defined($rv));
  my @result;
  while (my $row = $sth->fetchrow_hashref) {
    push @result, $row;
  }
  return @result;
}

sub psql_update {
  my ($this, $table, $set_array, $where) = @_;

  $n = 0;
  $query = "UPDATE " . $table . " SET ";
  my @vals;
  foreach $key (sort keys %$set_array) {
    if ($n > 0) {
      $query .= ", ";
    }
    $query .= "" . $key . "=?";
    push @vals, $set_array->{$key};
    $n++;
  }
  if ($where ne "") {
    $query .= " WHERE " . $where;
  }
  my $sth = $this->{'db'}->prepare($query);
  return $sth->execute(@vals);
}

return 1;
